#include <imsls.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
void print_error_rates(int classErrors[]);
int main(){                                
   int i, j;
   /* Inputs assuming all attributes, except family history, 
   are continuous */
   int n_patterns;       /* 4601 */
   int n_variables;      /* 57 + 1 classification */
   int n_classes       =  2; /* (spam or no spam) */
   int n_continuous    = 57;
   int *classErrors    = NULL;
   int *classification = NULL;
   int selected_pdf[57];
   /* additional double variables */
   float *continuous, *unscaledContinuous;
   float *spamData;
   int n_spam = 0;
   static char *fmt = "%10.2f";

   imsls_omp_options(IMSLS_SET_FUNCTIONS_THREAD_SAFE, 1, 0);

   spamData = imsls_f_data_sets(11, IMSLS_N_OBSERVATIONS, &n_patterns, 
      IMSLS_N_VARIABLES, &n_variables, 0);

   continuous = (float*) malloc( n_patterns * (n_variables-1) 
      * sizeof(float));
   unscaledContinuous = (float*) malloc( n_patterns * (n_variables-1) 
      * sizeof(float));
   classification = (int*) malloc( n_patterns*sizeof(int) );


   for(i=0; i<n_patterns; i++){
      for(j=0; j<(n_variables-1); j++) {
         if (j<54) {
            continuous[i*(n_variables-1)+j] = (float)
               asin(sqrt( spamData[i*n_variables+j]/100));
         } else {
            continuous[i*(n_variables-1)+j] = 
               spamData[i*n_variables+j];
         }
         unscaledContinuous[i*(n_variables-1)+j] = 
            spamData[i*n_variables+j];

      }
      classification[i] = (int)spamData[(i*n_variables)+n_variables-1];
      if(classification[i] == 1) n_spam++;
   }

   printf("Number of Patterns = %d \n", n_patterns);
   printf(" Number Classified as Spam = %d \n\n", n_spam);

   classErrors = imsls_f_naive_bayes_trainer(n_patterns, n_classes, 
      classification, 
      IMSLS_CONTINUOUS, n_continuous, unscaledContinuous, 0);
   printf("    Unscaled Gaussian Classification Error Rates \n");
   printf("           No Attribute Transformations          \n");
   printf("     All Attributes Modeled as Gaussian Variates.\n");
   print_error_rates(classErrors);
   imsls_free(classErrors);

   classErrors = imsls_f_naive_bayes_trainer(n_patterns, n_classes,
      classification, IMSLS_CONTINUOUS, n_continuous, continuous, 0);
   printf("    Scaled Gaussian Classification Error Rates  \n");
   printf("   Arsin(sqrt) transformation of first 54 Vars. \n");
   printf("   All Attributes Modeled as Gaussian Variates. \n");
   print_error_rates(classErrors);
   imsls_free(classErrors);

   for(i=0; i<54; i++){
      selected_pdf[i] = IMSLS_GAUSSIAN;
   }
   for(i=54; i<57; i++){
      selected_pdf[i] = IMSLS_LOG_NORMAL;
   }
   classErrors = imsls_f_naive_bayes_trainer(n_patterns, n_classes, 
      classification, 
      IMSLS_CONTINUOUS, n_continuous, continuous,
      IMSLS_SELECTED_PDF, selected_pdf, 0);
   printf("  Gaussian/Log Normal Classification Error Rates   \n");
   printf("  Arsin(sqrt) transformation of 1st 54 Attributes. \n");
   printf(" Gaussian - 1st 54 & Log Normal - last 3 Attributes\n");
   print_error_rates(classErrors);
   imsls_free(classErrors);

   /* scale continuous attributes using z-score scaling */
   for(i=0; i<n_patterns; i++){
      for(j=54; j<57; j++) continuous[i*n_continuous+j] = (float)
         sqrt(unscaledContinuous[i*n_continuous+j]);
   }
   for(i=0; i<57; i++){
      selected_pdf[i] = IMSLS_GAUSSIAN;
   }
   classErrors = imsls_f_naive_bayes_trainer(n_patterns, n_classes, 
      classification, 
      IMSLS_CONTINUOUS, n_continuous, continuous,
      IMSLS_SELECTED_PDF, selected_pdf, 0);
   printf("       Scaled Classification Error Rates         \n");
   printf("  Arsin(sqrt) transformation of 1st 54 Attributes\n");
   printf("    sqrt() transformation for last 3 Attributes  \n");
   printf("   All Attributes Modeled as Gaussian Variates.  \n");
   print_error_rates(classErrors);
   imsls_free(classErrors);

   for(i=54; i<57; i++){
      selected_pdf[i] = IMSLS_LOG_NORMAL;
   }
   classErrors = imsls_f_naive_bayes_trainer(n_patterns, n_classes, 
      classification, IMSLS_CONTINUOUS, n_continuous, continuous,
      IMSLS_SELECTED_PDF, selected_pdf, 0);
   printf("      Scaled Classification Error Rates\n");
   printf("  Arsin(sqrt) transformation of 1st 54 Attributes  \n");
   printf("  and sqrt() transformation for last 3 Attributes  \n");
   printf(" Gaussian - 1st 54 & Log Normal - last 3 Attributes\n");
   print_error_rates(classErrors);
   imsls_free(classErrors);

}

void print_error_rates(int classErrors[]){
   float p0, p1, p2;
   p0 = (float)100.0*classErrors[0]/classErrors[1];
   p1 = (float)100.0*classErrors[2]/classErrors[3];
   p2 = (float)100.0*classErrors[4]/classErrors[5];
   printf("----------------------------------------------------\n");
   printf("    Not Spam          Spam        |    TOTAL\n");
   printf(" %d/%d=%4.1f%%   %d/%d=%4.1f%%   | %d/%d=%4.1f%%\n", 
      classErrors[0], classErrors[1],
      p0, classErrors[2], classErrors[3],
      p1, classErrors[4], classErrors[5], p2);
   printf("----------------------------------------------------\n\n");
}
